/* 
 * %Z%%W% %I%
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 * IBM Java(tm)2 SDK, Standard Edition, v 1.4 
 *
 * (C) Copyright IBM Corp. 1998 , 2002. All Rights Reserved
 * (C) Copyright Sun Microsystems Inc. 1997 , 2002. All Rights Reserved
 * ===========================================================================
 */


package com.sun.tools.example.debug.bdi;

import com.sun.jdi.*;
import com.sun.jdi.request.*;

import java.util.*;

class EventRequestSpecList {
    
    // all specs
    private List eventRequestSpecs = Collections.synchronizedList(
                                                  new ArrayList());

    final ExecutionManager runtime;

    EventRequestSpecList(ExecutionManager runtime) {
	this.runtime = runtime;
    }

    /** 
     * Resolve all deferred eventRequests waiting for 'refType'.
     */
    void resolve(ReferenceType refType) {
        synchronized(eventRequestSpecs) {
            Iterator iter = eventRequestSpecs.iterator();
            while (iter.hasNext()) {
                ((EventRequestSpec)iter.next()).attemptResolve(refType);
             }
        }
    }

    void install(EventRequestSpec ers, VirtualMachine vm) {
        synchronized (eventRequestSpecs) {
            eventRequestSpecs.add(ers);
        }
        if (vm != null) {
            ers.attemptImmediateResolve(vm);
        }
    }
        
    BreakpointSpec 
    createClassLineBreakpoint(String classPattern, int line) {
        ReferenceTypeSpec refSpec = 
            new PatternReferenceTypeSpec(classPattern);
        return new LineBreakpointSpec(this, refSpec, line);
    }
        
    BreakpointSpec 
    createSourceLineBreakpoint(String sourceName, int line) {
        ReferenceTypeSpec refSpec = 
            new SourceNameReferenceTypeSpec(sourceName, line);
        return new LineBreakpointSpec(this, refSpec, line);
    }
        
    BreakpointSpec 
    createMethodBreakpoint(String classPattern, 
                           String methodId, List methodArgs) {
        ReferenceTypeSpec refSpec = 
            new PatternReferenceTypeSpec(classPattern);
        return new MethodBreakpointSpec(this, refSpec, 
                                        methodId, methodArgs);
    }
        
    ExceptionSpec 
    createExceptionIntercept(String classPattern,
                             boolean notifyCaught,
                             boolean notifyUncaught) {
        ReferenceTypeSpec refSpec = 
            new PatternReferenceTypeSpec(classPattern);
        return new ExceptionSpec(this, refSpec,
                                 notifyCaught, notifyUncaught);
    }
        
    AccessWatchpointSpec 
    createAccessWatchpoint(String classPattern, String fieldId) {
        ReferenceTypeSpec refSpec = 
            new PatternReferenceTypeSpec(classPattern);
        return new AccessWatchpointSpec(this, refSpec, fieldId);
    }
        
    ModificationWatchpointSpec 
    createModificationWatchpoint(String classPattern, String fieldId) {
        ReferenceTypeSpec refSpec = 
            new PatternReferenceTypeSpec(classPattern);
        return new ModificationWatchpointSpec(this, refSpec, fieldId);
    }

    void delete(EventRequestSpec ers) {
        EventRequest request = ers.getEventRequest();
        synchronized (eventRequestSpecs) {
            eventRequestSpecs.remove(ers);
        }
        if (request != null) {
            request.virtualMachine().eventRequestManager()
                .deleteEventRequest(request);
        }
        notifyDeleted(ers);
        //### notify delete - here?
    }

    List eventRequestSpecs() {
        // We need to make a copy to avoid synchronization problems
        synchronized (eventRequestSpecs) {
            return new ArrayList(eventRequestSpecs);
        }
    }

    // --------  notify routines --------------------

    private Vector specListeners() {
        return (Vector)runtime.specListeners.clone();
    }

    void notifySet(EventRequestSpec spec) {
	Vector l = specListeners();
	SpecEvent evt = new SpecEvent(spec);
	for (int i = 0; i < l.size(); i++) {
	    spec.notifySet((SpecListener)l.elementAt(i), evt);
	}
    }

    void notifyDeferred(EventRequestSpec spec) {
	Vector l = specListeners();
	SpecEvent evt = new SpecEvent(spec);
	for (int i = 0; i < l.size(); i++) {
	    spec.notifyDeferred((SpecListener)l.elementAt(i), evt);
	}
    }

    void notifyDeleted(EventRequestSpec spec) {
	Vector l = specListeners();
	SpecEvent evt = new SpecEvent(spec);
	for (int i = 0; i < l.size(); i++) {
	    spec.notifyDeleted((SpecListener)l.elementAt(i), evt);
	}
    }

    void notifyResolved(EventRequestSpec spec) {
	Vector l = specListeners();
	SpecEvent evt = new SpecEvent(spec);
	for (int i = 0; i < l.size(); i++) {
	    spec.notifyResolved((SpecListener)l.elementAt(i), evt);
	}
    }

    void notifyError(EventRequestSpec spec, Exception exc) {
	Vector l = specListeners();
	SpecErrorEvent evt = new SpecErrorEvent(spec, exc);
	for (int i = 0; i < l.size(); i++) {
	    spec.notifyError((SpecListener)l.elementAt(i), evt);
	}
    }
}



